﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Diagnostics;
using Telerik.Windows.Controls;

namespace RadControlsWpfApp1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class NumberSpinner : UserControl
    {

        public static readonly DependencyProperty FocusedBackgroundProperty =
              DependencyProperty.Register("FocusedBackground", typeof(Brush), typeof(NumberSpinner), new PropertyMetadata(null));

        public static readonly DependencyProperty FocusedForegroundProperty =
            DependencyProperty.Register("FocusedForeground", typeof(Brush), typeof(NumberSpinner), new PropertyMetadata(null));

        public static DependencyProperty GridBackgroundProperty =
            DependencyProperty.Register("GridBackground", typeof(Brush), typeof(NumberSpinner), new PropertyMetadata(Brushes.Transparent));

        public static readonly DependencyProperty IncrementProperty =
            DependencyProperty.Register("Increment", typeof(double), typeof(NumberSpinner), new PropertyMetadata(1.0));

        public static readonly DependencyProperty MaskProperty =
            DependencyProperty.Register("Mask", typeof(string), typeof(NumberSpinner), new PropertyMetadata("#9.2"));

        public static readonly DependencyProperty MaximumProperty =
            DependencyProperty.Register("Maximum", typeof(double), typeof(NumberSpinner),
                                             new PropertyMetadata(double.NaN, new PropertyChangedCallback(OnMaximumInvalidated)));

        public static readonly DependencyProperty MinimumProperty =
            DependencyProperty.Register("Minimum", typeof(double), typeof(NumberSpinner),
                                             new PropertyMetadata(double.NaN, new PropertyChangedCallback(OnMinimumInvalidated)));

        //public static readonly DependencyProperty TimeOfDayModeProperty =
        //    DependencyProperty.Register( "TimeOfDayMode", typeof( TimesOfDay ), typeof( NumberSpinner ),
        //                                 new FrameworkPropertyMetadata( TimesOfDay.DayTime,
        //                                     FrameworkPropertyMetadataOptions.AffectsRender,
        //                                     new PropertyChangedCallback( OnTimeOfDayInvalidated ) ) );

        public static readonly DependencyProperty UnfocusedBackgroundProperty =
            DependencyProperty.Register("UnfocusedBackground", typeof(Brush), typeof(NumberSpinner), new PropertyMetadata(null));

        public static readonly DependencyProperty UnfocusedForegroundProperty =
            DependencyProperty.Register("UnfocusedForeground", typeof(Brush), typeof(NumberSpinner), new PropertyMetadata(null));

        public static readonly DependencyProperty ValueProperty =
            DependencyProperty.Register("Value", typeof(double), typeof(NumberSpinner),
                                             new PropertyMetadata(0.0, new PropertyChangedCallback(OnValueInvalidated)));

        public Brush FocusedBackground
        {
            get { return (Brush)GetValue(FocusedBackgroundProperty); }
            set { SetValue(FocusedBackgroundProperty, value); }
        }

        public Brush FocusedForeground
        {
            get { return (Brush)GetValue(FocusedForegroundProperty); }
            set { SetValue(FocusedForegroundProperty, value); }
        }

        public Brush GridBackground
        {
            get { return (Brush)GetValue(GridBackgroundProperty); }
            set { SetValue(GridBackgroundProperty, value); }
        }

        public double Increment
        {
            get { return (double)GetValue(IncrementProperty); }
            set { SetValue(IncrementProperty, value); }
        }

        public string Mask
        {
            get { return (string)GetValue(MaskProperty); }
            set { SetValue(MaskProperty, value); }
        }

        public double Maximum
        {
            get { return (double)GetValue(MaximumProperty); }
            set { SetValue(MaximumProperty, value); }
        }

        public double Minimum
        {
            get { return (double)GetValue(MinimumProperty); }
            set { SetValue(MinimumProperty, value); }
        }

        //public TimesOfDay TimeOfDayMode {
        //    get { return (TimesOfDay) GetValue( TimeOfDayModeProperty ); }
        //    set { SetValue( TimeOfDayModeProperty, value ); }
        //}

        public Brush UnfocusedBackground
        {
            get { return (Brush)GetValue(UnfocusedBackgroundProperty); }
            set { SetValue(UnfocusedBackgroundProperty, value); }
        }

        public Brush UnfocusedForeground
        {
            get { return (Brush)GetValue(UnfocusedForegroundProperty); }
            set { SetValue(UnfocusedForegroundProperty, value); }
        }

        public double Value
        {
            get { return (double)GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        public NumberSpinner()
        {
            InitializeComponent();
        }

        private void DecrementButton_Click(object sender, RoutedEventArgs e)
        {
            Value -= Increment;
        }

        private void IncrementButton_Click(object sender, RoutedEventArgs e)
        {
            Value += Increment;
        }

        private void OnMaximumChanged(double newMaximum)
        {
            // Does the control have a Maximum value?
            if (!double.IsNaN(newMaximum))
            {
                // It does.  Is the value less than or equal to the maximum?
                if (Value > newMaximum)
                {
                    // It is not.  Set the Value to the newMaximum.
                    Value = newMaximum;
                }
                IncrementButton.IsEnabled = Value < newMaximum;
            }
        }

        private static void OnMaximumInvalidated(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            NumberSpinner spinner = d as NumberSpinner;
            spinner.OnMaximumChanged((double)e.NewValue);
        }

        private void OnMinimumChanged(double newMinimum)
        {
            if (!double.IsNaN(newMinimum))
            {
                if (Value < newMinimum)
                {
                    Value = newMinimum;
                }

                DecrementButton.IsEnabled = Value > newMinimum;
            }
        }

        private static void OnMinimumInvalidated(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            NumberSpinner spinner = d as NumberSpinner;
            spinner.OnMinimumChanged((double)e.NewValue);
        }

        //private void OnTimeOfDayModeChanged( TimesOfDay newTimeofDayMode ) {
        //    if ( FocusedBackground != null && UnfocusedBackground != null ) {
        //        ValueBox.Background = ValueBox.IsFocused ? FocusedBackground : UnfocusedBackground;
        //    }

        //    if ( FocusedForeground != null && UnfocusedForeground != null ) {
        //        ValueBox.Foreground = ValueBox.IsFocused ? FocusedForeground : UnfocusedForeground;
        //    }
        //}

        private static void OnTimeOfDayInvalidated(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            NumberSpinner spinner = (NumberSpinner)d;
            //spinner.OnTimeOfDayModeChanged( (TimesOfDay) e.NewValue );
        }

        private void OnValueChanged(double newValue)
        {
            if (!double.IsNaN(Minimum))
            {
                if (newValue < Minimum)
                {
                    Value = Minimum;
                }

                DecrementButton.IsEnabled = Value > Minimum;
            }

            if (!double.IsNaN(Maximum))
            {
                if (Value > Maximum)
                {
                    Value = Maximum;
                }

                IncrementButton.IsEnabled = Value < Maximum;
            }
        }

        private static void OnValueInvalidated(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            NumberSpinner spinner = d as NumberSpinner;
            spinner.OnValueChanged((double)e.NewValue);
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            if (FocusedBackground == null && UnfocusedBackground == null)
            {
                ValueBox.Background = Background;
            }

            if (FocusedForeground == null && UnfocusedForeground == null)
            {
                ValueBox.Foreground = Foreground;
            }
        }

        private void ValueBox_GotFocus(object sender, RoutedEventArgs e)
        {
            if (FocusedBackground != null)
            {
                ValueBox.Background = FocusedBackground;
            }

            if (FocusedForeground != null)
            {
                ValueBox.Foreground = FocusedForeground;
            }
        }

        private void ValueBox_LostFocus(object sender, RoutedEventArgs e)
        {
            if (UnfocusedBackground != null)
            {
                ValueBox.Background = UnfocusedBackground;
            }

            if (UnfocusedForeground != null)
            {
                ValueBox.Foreground = UnfocusedForeground;
            }
        }
    }
}

